import _createClass from "@babel/runtime/helpers/createClass";
import { TransformStage, GlobalEvent } from '@alilc/lowcode-types';
import Node from './node';
import Selection from './selection';
import Detecting from './detecting';
import History from './history';
import Project from './project';
import Prop from './prop';
import Canvas from './canvas';
import { documentSymbol, editorSymbol, nodeSymbol } from './symbols';

var DocumentModel = /*#__PURE__*/function () {
  function DocumentModel(document) {
    this[documentSymbol] = void 0;
    this[editorSymbol] = void 0;
    this.selection = void 0;
    this.detecting = void 0;
    this.history = void 0;
    this.canvas = void 0;
    this[documentSymbol] = document;
    this[editorSymbol] = document.designer.editor;
    this.selection = new Selection(document);
    this.detecting = new Detecting(document);
    this.history = new History(document.getHistory());
    this.canvas = new Canvas(document.designer);
  }

  DocumentModel.create = function create(document) {
    if (document == undefined) return null;
    return new DocumentModel(document);
  }
  /**
   * 获取当前文档所属的 project
   * @returns
   */
  ;

  var _proto = DocumentModel.prototype;

  /**
   * 根据 nodeId 返回 Node 实例
   * @param nodeId
   * @returns
   */
  _proto.getNodeById = function getNodeById(nodeId) {
    return Node.create(this[documentSymbol].getNode(nodeId));
  }
  /**
   * 导入 schema
   * @param schema
   */
  ;

  _proto.importSchema = function importSchema(schema) {
    this[documentSymbol]["import"](schema);
  }
  /**
   * 导出 schema
   * @param stage
   * @returns
   */
  ;

  _proto.exportSchema = function exportSchema(stage) {
    if (stage === void 0) {
      stage = TransformStage.Render;
    }

    return this[documentSymbol]["export"](stage);
  }
  /**
   * 插入节点
   * @param parent
   * @param thing
   * @param at
   * @param copy
   * @returns
   */
  ;

  _proto.insertNode = function insertNode(parent, thing, at, copy) {
    var node = this[documentSymbol].insertNode(parent[nodeSymbol], thing === null || thing === void 0 ? void 0 : thing[nodeSymbol], at, copy);
    return Node.create(node);
  }
  /**
   * 创建一个节点
   * @param data
   * @returns
   */
  ;

  _proto.createNode = function createNode(data) {
    return Node.create(this[documentSymbol].createNode(data));
  }
  /**
   * 移除指定节点/节点id
   * @param idOrNode
   */
  ;

  _proto.removeNode = function removeNode(idOrNode) {
    this[documentSymbol].removeNode(idOrNode);
  }
  /**
   * 当前 document 新增节点事件
   */
  ;

  _proto.onAddNode = function onAddNode(fn) {
    this[documentSymbol].onNodeCreate(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 删除节点事件
   */
  ;

  _proto.onRemoveNode = function onRemoveNode(fn) {
    this[documentSymbol].onNodeDestroy(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 的 hover 变更事件
   */
  ;

  _proto.onChangeDetecting = function onChangeDetecting(fn) {
    this[documentSymbol].designer.detecting.onDetectingChange(function (node) {
      fn(Node.create(node));
    });
  }
  /**
   * 当前 document 的选中变更事件
   */
  ;

  _proto.onChangeSelection = function onChangeSelection(fn) {
    this[documentSymbol].selection.onSelectionChange(function (ids) {
      fn(ids);
    });
  }
  /**
   * 当前 document 的节点显隐状态变更事件
   * @param fn
   */
  ;

  _proto.onChangeNodeVisible = function onChangeNodeVisible(fn) {
    // TODO: history 变化时需要重新绑定
    this[documentSymbol].nodesMap.forEach(function (node) {
      node.onVisibleChange(function (flag) {
        fn(Node.create(node), flag);
      });
    });
  }
  /**
   * 当前 document 的节点 children 变更事件
   * @param fn
   */
  ;

  _proto.onChangeNodeChildren = function onChangeNodeChildren(fn) {
    // TODO: history 变化时需要重新绑定
    this[documentSymbol].nodesMap.forEach(function (node) {
      node.onChildrenChange(function (info) {
        return info ? fn({
          type: info.type,
          node: Node.create(node)
        }) : fn();
      });
    });
  }
  /**
   * 当前 document 节点属性修改事件
   * @param fn
   */
  ;

  _proto.onChangeNodeProp = function onChangeNodeProp(fn) {
    this[editorSymbol].on(GlobalEvent.Node.Prop.InnerChange, function (info) {
      fn({
        key: info.key,
        oldValue: info.oldValue,
        newValue: info.newValue,
        prop: Prop.create(info.prop),
        node: Node.create(info.node)
      });
    });
  };

  _createClass(DocumentModel, [{
    key: "project",
    get: function get() {
      return Project.create(this[documentSymbol].project);
    }
    /**
     * 获取文档的根节点
     * @returns
     */

  }, {
    key: "root",
    get: function get() {
      return Node.create(this[documentSymbol].getRoot());
    }
    /**
     * 获取文档下所有节点
     * @returns
     */

  }, {
    key: "nodesMap",
    get: function get() {
      var map = new Map();

      for (var id in this[documentSymbol].nodesMap.keys()) {
        map.set(id, this.getNodeById(id));
      }

      return map;
    }
  }]);

  return DocumentModel;
}();

export { DocumentModel as default };