import { getLogger } from '@alilc/lowcode-utils';
import { editorSymbol } from './symbols';
var logger = getLogger({
  level: 'warn',
  bizName: 'shell:event'
});

var Event = /*#__PURE__*/function () {
  // TODO:

  /**
   * 内核触发的事件名
   */
  function Event(editor, options) {
    this[editorSymbol] = void 0;
    this.options = void 0;
    this.names = [];
    this[editorSymbol] = editor;
    this.options = options;

    if (!this.options.prefix) {
      logger.warn('prefix is required while initializing Event');
    }
  }
  /**
   * 监听事件
   * @param event 事件名称
   * @param listener 事件回调
   */


  var _proto = Event.prototype;

  _proto.on = function on(event, listener) {
    if (event.startsWith('designer')) {
      logger.warn('designer events are disabled');
      return;
    }

    this[editorSymbol].on(event, listener);
  }
  /**
   * 取消监听事件
   * @param event 事件名称
   * @param listener 事件回调
   */
  ;

  _proto.off = function off(event, listener) {
    this[editorSymbol].off(event, listener);
  }
  /**
   * 触发事件
   * @param event 事件名称
   * @param args 事件参数
   * @returns
   */
  ;

  _proto.emit = function emit(event) {
    var _this$editorSymbol;

    if (!this.options.prefix) {
      logger.warn('Event#emit has been forbidden while prefix is not specified');
      return;
    }

    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    (_this$editorSymbol = this[editorSymbol]).emit.apply(_this$editorSymbol, [this.options.prefix + ":" + event].concat(args));
  };

  return Event;
}();

export { Event as default };
export function getEvent(editor, options) {
  if (options === void 0) {
    options = {
      prefix: 'common'
    };
  }

  return new Event(editor, options);
}