"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _documentModel = _interopRequireDefault(require("./document-model"));

var _simulatorHost = _interopRequireDefault(require("./simulator-host"));

var _symbols = require("./symbols");

var Project = /*#__PURE__*/function () {
  function Project(project) {
    this[_symbols.projectSymbol] = void 0;
    this[_symbols.simulatorHostSymbol] = void 0;
    this[_symbols.simulatorRendererSymbol] = void 0;
    this[_symbols.projectSymbol] = project;
  }

  Project.create = function create(project) {
    return new Project(project);
  }
  /**
   * 获取当前的 document
   * @returns
   */
  ;

  var _proto = Project.prototype;

  /**
   * 打开一个 document
   * @param doc
   * @returns
   */
  _proto.openDocument = function openDocument(doc) {
    var documentModel = this[_symbols.projectSymbol].open(doc);

    if (!documentModel) return null;
    return _documentModel["default"].create(documentModel);
  }
  /**
   * 创建一个 document
   * @param data
   * @returns
   */
  ;

  _proto.createDocument = function createDocument(data) {
    var doc = this[_symbols.projectSymbol].createDocument(data);

    return _documentModel["default"].create(doc);
  }
  /**
   * 根据 fileName 获取 document
   * @param fileName
   * @returns
   */
  ;

  _proto.getDocumentByFileName = function getDocumentByFileName(fileName) {
    return _documentModel["default"].create(this[_symbols.projectSymbol].getDocumentByFileName(fileName));
  }
  /**
   * 根据 id 获取 document
   * @param id
   * @returns
   */
  ;

  _proto.getDocumentById = function getDocumentById(id) {
    return _documentModel["default"].create(this[_symbols.projectSymbol].getDocument(id));
  }
  /**
   * 导出 project
   * @returns
   */
  ;

  _proto.exportSchema = function exportSchema() {
    return this[_symbols.projectSymbol].getSchema();
  }
  /**
   * 导入 project
   * @param schema 待导入的 project 数据
   */
  ;

  _proto.importSchema = function importSchema(schema) {
    this[_symbols.projectSymbol].load(schema, true);
  }
  /**
   * 获取当前的 document
   * @returns
   */
  ;

  _proto.getCurrentDocument = function getCurrentDocument() {
    return _documentModel["default"].create(this[_symbols.projectSymbol].currentDocument);
  }
  /**
   * 增加一个属性的管道处理函数
   * @param transducer
   * @param stage
   */
  ;

  _proto.addPropsTransducer = function addPropsTransducer(transducer, stage) {
    this[_symbols.projectSymbol].designer.addPropsReducer(transducer, stage);
  }
  /**
   * 当前 project 内的 document 变更事件
   */
  ;

  _proto.onChangeDocument = function onChangeDocument(fn) {
    if (this[_symbols.projectSymbol].currentDocument) {
      fn(_documentModel["default"].create(this[_symbols.projectSymbol].currentDocument));
      return function () {};
    }

    return this[_symbols.projectSymbol].onCurrentDocumentChange(function (originalDoc) {
      fn(_documentModel["default"].create(originalDoc));
    });
  }
  /**
   * 当前 project 的模拟器 ready 事件
   */
  ;

  _proto.onSimulatorHostReady = function onSimulatorHostReady(fn) {
    var _this = this;

    if (this[_symbols.simulatorHostSymbol]) {
      fn(_simulatorHost["default"].create(this[_symbols.simulatorHostSymbol]));
      return function () {};
    }

    return this[_symbols.projectSymbol].onSimulatorReady(function (simulator) {
      _this[_symbols.simulatorHostSymbol] = simulator;
      fn(_simulatorHost["default"].create(simulator));
    });
  }
  /**
   * 当前 project 的渲染器 ready 事件
   */
  ;

  _proto.onSimulatorRendererReady = function onSimulatorRendererReady(fn) {
    var _this2 = this;

    if (this[_symbols.simulatorRendererSymbol]) {
      fn();
      return function () {};
    } // TODO: 补充 renderer 实例


    return this[_symbols.projectSymbol].onRendererReady(function (renderer) {
      _this2[_symbols.simulatorRendererSymbol] = renderer;
      fn();
    });
  };

  (0, _createClass2["default"])(Project, [{
    key: "currentDocument",
    get: function get() {
      return this.getCurrentDocument();
    }
    /**
     * 获取当前 project 下所有 documents
     * @returns
     */

  }, {
    key: "documents",
    get: function get() {
      return this[_symbols.projectSymbol].documents.map(function (doc) {
        return _documentModel["default"].create(doc);
      });
    }
    /**
     * 获取模拟器的 host
     */

  }, {
    key: "simulatorHost",
    get: function get() {
      return _simulatorHost["default"].create(this[_symbols.projectSymbol].simulator || this[_symbols.simulatorHostSymbol]);
    }
    /**
     * @deprecated use .simulatorHost instead.
     */

  }, {
    key: "simulator",
    get: function get() {
      return this.simulatorHost;
    }
  }]);
  return Project;
}();

exports["default"] = Project;