import { Props as InnerProps } from '@alilc/lowcode-designer';
import { CompositeValue } from '@alilc/lowcode-types';
import { propsSymbol } from './symbols';
import Node from './node';
import Prop from './prop';
export default class Props {
    private readonly [propsSymbol];
    constructor(props: InnerProps);
    static create(props: InnerProps | undefined | null): Props;
    /**
     * id
     */
    get id(): string;
    /**
     * 返回当前 props 的路径
     */
    get path(): any[];
    get node(): Node | null;
    /**
     * 获取指定 path 的属性模型实例
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @returns
     */
    getProp(path: string): Prop | null;
    /**
     * 获取指定 path 的属性模型实例值
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @returns
     */
    getPropValue(path: string): CompositeValue;
    /**
     * 获取指定 path 的属性模型实例，
     *  注：导出时，不同于普通属性，该属性并不挂载在 props 之下，而是与 props 同级
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @returns
     */
    getExtraProp(path: string): Prop | null;
    /**
     * 获取指定 path 的属性模型实例，
     *  注：导出时，不同于普通属性，该属性并不挂载在 props 之下，而是与 props 同级
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @returns
     */
    getExtraPropValue(path: string): CompositeValue;
    /**
     * 设置指定 path 的属性模型实例值
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @param value 值
     * @returns
     */
    setPropValue(path: string, value: CompositeValue): void;
    /**
     * 设置指定 path 的属性模型实例值
     * @param path 属性路径，支持 a / a.b / a.0 等格式
     * @param value 值
     * @returns
     */
    setExtraPropValue(path: string, value: CompositeValue): void;
}
