import { SettingField } from '@alilc/lowcode-designer';
import { CompositeValue, FieldConfig } from '@alilc/lowcode-types';
import { settingPropEntrySymbol } from './symbols';
import Node from './node';
export default class SettingPropEntry {
    private readonly [settingPropEntrySymbol];
    constructor(prop: SettingField);
    static create(prop: SettingField): SettingPropEntry;
    /**
     * 获取设置属性的 name
     */
    get name(): string | number;
    /**
     * 获取设置属性的 key
     */
    get key(): string | number;
    /**
     * 获取设置属性对应的节点实例
     */
    get node(): Node | null;
    /**
     * 获取设置属性的父设置属性
     */
    get parent(): SettingPropEntry;
    /**
     * @deprecated use .node instead
     */
    getNode(): Node;
    /**
     * @deprecated use .parent instead
     */
    getParent(): SettingPropEntry;
    /**
     * 设置值
     * @param val 值
     */
    setValue(val: CompositeValue): void;
    /**
     * 设置子级属性值
     * @param propName 子属性名
     * @param value 值
     */
    setPropValue(propName: string | number, value: any): void;
    /**
     * 获取值
     * @returns
     */
    getValue(): any;
    /**
     * 获取设置属性集
     * @returns
     */
    getProps(): any;
    /**
     * 是否绑定了变量
     * @returns
     */
    isUseVariable(): boolean;
    /**
     * 设置绑定变量
     * @param flag
     */
    setUseVariable(flag: boolean): void;
    /**
     * 创建一个设置 field 实例
     * @param config
     * @returns
     */
    createField(config: FieldConfig): SettingPropEntry;
    /**
     * 获取值，当为变量时，返回 mock
     * @returns
     */
    getMockOrValue(): any;
}
